<?php
/**
 * Mobile Basic template for Mobile Joomla! extension
 * https://www.mobilejoomla.com
 *
 * @version     3.2.2
 * @license     GNU/GPL v2 - http://www.gnu.org/licenses/gpl-2.0.html
 * @copyright   (C) 2008-2024 Kuneri Ltd. / Denis Ryabov
 * @date        February 2024
 */
defined('_JEXEC') or die('Restricted access');

class MJJqmFramework
{
    /** @var string */
    private $baseurl;
    /** @var string */
    private $template;
    /** @var Joomla\Registry\Registry */
    private $params;
    /** @var string */
    private $base;
    /** @var string */
    private $path;
    /** @var bool */
    private $rtl;
    /** @var bool */
    private $debug = false;

    /** @var string jQueryMobile version */
    public $jqm_ver = '1.4.5';
    /** @var string jQuery version */
    public $jq_ver = '1.9.1';
    /** @var bool Load jQuery Migration */
    public $jq_migration = false;
    /** @var string jQuery Migration version */
    public $jq_migration_ver = '1.2.1';
    /** @var string Theme name */
    private $themeroller = ''; // @todo Is it really unused???

    /** @var string */
    private $themeroller_css = '';
    /** @var string */
    private $themeroller_icons = '';
    /** @var string */
    private $themeroller_custom;
    /** @var string */
    private $jqm_css = '';
    /** @var string */
    private $jqm_jq = '';
    /** @var string */
    private $jqm_jqmigr = '';
    /** @var string */
    private $jqm_jqm = '';

    /** @var array */
    private $loaded_css = array();
    /** @var array */
    private $loaded_js = array();
    /** @var array */
    private $combinerSource = array();

    /** @var string */
    private $exclude_css = '';
    /** @var string */
    private $exclude_js = '';

    /** @var bool */
    private $compatJQueryEasy = false;
    /** @var bool */
    private $compatJQueryEasySafeNoConflict = false; // use noConflict( ) to don't match jQueryEasy regexp

    /**
     * @param $document JDocumentHTML
     */
    public function __construct($document)
    {
        MJFactory::getApplication()->registerEvent('onMJRessioPrepare', array($this, 'onMJRessioPrepare'));

        $this->baseurl = $document->baseurl;
        $this->template = $document->template;
        $this->params = $document->params;
        $this->base = '/templates/' . $this->template . '/';
        $this->path = dirname(__DIR__);
        $this->rtl = ($document->direction === 'rtl');

        $this->debug = $this->debug || JDEBUG || $this->params->get('debug');

        $this->initURLs();
    }

    /**
     * @param $document ?JDocumentHTML
     * @return self
     */
    public static function getInstance($document = null)
    {
        static $instance;
        if (!isset($instance) && isset($document)) {
            $instance = new self($document);
        }
        return $instance;
    }

    public function prepareHead()
    {
        $app = MJFactory::getApplication();

        /** @var $doc JDocumentHTML */
        $doc = MJFactory::getDocument();

        $app->triggerEvent('onBeforeCompileHead', array('subject' => $app, 'document' => $doc));

        $headerstuff = $doc->getHeadData();

        // Force Mootools loading
        if ($this->params->get('mootools')) {
            $this->forceMootools($headerstuff);
        }

        // populate preload_styleSheets and preload_scripts
        $this->parseCustomPreload($headerstuff);

        $this->fixJQueryEasy($headerstuff);
        if ($this->params->get('fixbootstrap')) {
            $this->fixBootstrap($headerstuff);
        }
        if ($this->params->get('fixjqueryui')) {
            $this->fixJQueryUI($headerstuff);
        }
        if ($this->params->get('removejquery')) {
            $this->removeJQuery($headerstuff);
        }

        $this->generateHead($headerstuff);
    }

    private function parseCustomPreload(&$headerstuff)
    {
        $css_custom_preload = $this->path . '/css/custom_preload.txt';
        $js_custom_preload = $this->path . '/js/custom_preload.txt';

        $headerstuff['preload_styleSheets'] = array();
        $headerstuff['preload_scripts'] = array();

        if (is_file($css_custom_preload)) {
            $custom_styles = @file($css_custom_preload);
            foreach ($custom_styles as $url) {
                $url = trim($url);
                if ($url !== '') {
                    switch ($url[0]) {
                        case '#':
                            break;
                        case '-':
                            $url = ltrim(substr($url, 1));
                            if (strpos($url, '*') === false) {
                                $this->loaded_css[$url] = 1;
                            } else {
                                if ($this->exclude_css !== '') {
                                    $this->exclude_css .= '|';
                                }
                                $this->exclude_css .= str_replace('\*', '.*', preg_quote($url, '/'));
                            }
                            break;
                        case '*':
                            $url = ltrim(substr($url, 1));
                            $headerstuff['preload_styleSheets'][$url] = 1;
                            break;
                        default:
                            if (!isset($headerstuff['styleSheets'][$url])) {
                                $headerstuff['styleSheets'][$url] = array('mime' => 'text/css', 'media' => null, 'attribs' => array());
                            }
                    }
                }
            }
        }

        if (is_file($js_custom_preload)) {
            $custom_scripts = @file($js_custom_preload);
            foreach ($custom_scripts as $url) {
                $url = trim($url);
                if ($url !== '') {
                    switch ($url[0]) {
                        case '#':
                            break;
                        case '-':
                            $url = ltrim(substr($url, 1));
                            if (strpos($url, '*') === false) {
                                $this->loaded_js[$url] = 1;
                            } else {
                                if ($this->exclude_js !== '') {
                                    $this->exclude_js .= '|';
                                }
                                $this->exclude_js .= str_replace('\*', '.*', preg_quote($url, '/'));
                            }
                            break;
                        case '*':
                            $url = ltrim(substr($url, 1));
                            $headerstuff['preload_scripts'][$url] = 1;
                            break;
                        default:
                            if (!isset($headerstuff['scripts'][$url])) {
                                $headerstuff['scripts'][$url] = array('mime' => 'text/javascript', 'defer' => false, 'async' => false);
                            }
                    }
                }
            }
        }
    }

    /**
     * @param $headerstuff array
     */
    private function fixJQueryEasy(&$headerstuff)
    {
        $plugin = MJPluginHelper::getPlugin('system', 'jqueryeasy');
        if (count($plugin) === 0) {
            return;
        }

        /** $param Joomla\Registry\Registry */
        $params = new MJRegistry($plugin->params);

        if (!$params->get('jqueryinfrontend', 0)) {
            return;
        }

        //$this->params->set('load_external', 1); // for correct work of combiner
        $this->compatJQueryEasy = true;

        if ($params->get('removenoconflictfrontend', 1)) {
            $this->compatJQueryEasySafeNoConflict = true;
        }

        if (isset($headerstuff['scripts']['JQLIB'])) {
            unset($headerstuff['scripts']['JQLIB']);
            $this->jqm_jq = 'JQLIB';
        }
        if (isset($headerstuff['scripts']['JQEASY_JQLIB'])) {
            unset($headerstuff['scripts']['JQEASY_JQLIB']);
            $this->jqm_jq = 'JQEASY_JQLIB';
        }
        if (isset($headerstuff['scripts']['JQNOCONFLICT'])) {
            unset($headerstuff['scripts']['JQNOCONFLICT']);
        }
        if (isset($headerstuff['scripts']['JQEASY_JQNOCONFLICT'])) {
            unset($headerstuff['scripts']['JQEASY_JQNOCONFLICT']);
        }
        if (isset($headerstuff['script']['text/javascript'])) {
            $headerstuff['script']['text/javascript'] = str_replace(array('JQEASY_JQNOCONFLICT', 'JQNOCONFLICT'), array('', ''), $headerstuff['script']['text/javascript']);
        }
    }

    /**
     * @param $headerstuff array
     */
    private function fixBootstrap(&$headerstuff)
    {
        $regexp = '/bootstrap(?:[\.-][\w\.-]*)\.css';
        foreach ($headerstuff['styleSheets'] as $url => $param) {
            if (preg_match('#' . $regexp . '#', $url)) {
                $headerstuff['preload_styleSheets'][$url] = 1;
            }
        }
        if (count($headerstuff['custom'])) {
            foreach ($headerstuff['custom'] as &$custom) {
                if (preg_match_all('#<link\s[^>]*href="([^">]*?' . $regexp . '[^">]*?)"[^>]*/?>#', $custom, $matches)) {
                    foreach ($matches[1] as $index => $url) {
                        $headerstuff['preload_styleSheets'][$url] = 1;
                        $custom = str_replace($matches[0][$index], '', $custom);
                    }
                }
                $custom = trim($custom);
            }
            unset($custom);
        }

        $embedded_code = 'jQuery.fn.button.noConflict()';

        $app = MJFactory::getApplication();
        $mj_list = $app->triggerEvent('onGetMobileJoomla');
        /** @var MobileJoomla $mj */
        $mj = array_pop($mj_list);
        $isJoomla4 = version_compare(JVERSION, '4.0', '>=');
        if ($mj->getParam('js_mergeinline') || !$mj->getParam('js_merge')) {
            if (!isset($headerstuff['script']['text/javascript'])) {
                $headerstuff['script']['text/javascript'] = $isJoomla4 ? array() : '';
            }
            $code = "try{{$embedded_code}}catch(e){};";
            if ($isJoomla4) {
                $headerstuff['script']['text/javascript'][md5($code)] = $code;
            } else {
                $headerstuff['script']['text/javascript'] .= $code;
            }
        } else {
            if (!isset($headerstuff['custom'])) {
                $headerstuff['custom'] = array();
            }
            if ($mj->getParam('js_wraptrycatch')) {
                $headerstuff['custom'][] = "<script ress-merge>{$embedded_code}</script>";
            } else {
                $headerstuff['custom'][] = "<script ress-merge>try{{$embedded_code}}catch(e){}</script>";
            }
        }
    }

    /**
     * @param $headerstuff array
     */
    private function fixJQueryUI(&$headerstuff)
    {
        $regexp = '(?:/jquery-ui(?:[0-9\.-]|custom|min|pack)*?\.css|JQEASY_JQUICSS)';
        foreach ($headerstuff['styleSheets'] as $url => $param) {
            if (preg_match('#' . $regexp . '#', $url)) {
                $headerstuff['preload_styleSheets'][$url] = 1;
            }
        }
        if (count($headerstuff['custom'])) {
            foreach ($headerstuff['custom'] as &$custom) {
                if (preg_match_all('#<link\s[^>]*href="([^">]*?' . $regexp . '[^">]*?)"[^>]*/?>#', $custom, $matches)) {
                    foreach ($matches[1] as $index => $url) {
                        $headerstuff['preload_styleSheets'][$url] = 1;
                        $custom = str_replace($matches[0][$index], '', $custom);
                    }
                }
                $custom = trim($custom);
            }
        }
    }

    /**
     * @param $headerstuff array
     */
    private function removeJQuery(&$headerstuff)
    {
        $regexp = '/jquery(?:[0-9\.-]|latest|min|pack)*?\.js';
        foreach ($headerstuff['scripts'] as $url => $param) {
            if (preg_match('#' . $regexp . '#', $url)) {
                $this->loaded_js[$url] = 1;
            }
        }
        if (count($headerstuff['custom'])) {
            foreach ($headerstuff['custom'] as &$custom) {
                if (preg_match_all('#<script\s[^>]*src="([^">]*' . $regexp . '[^">]*)"[^>]*></script>#', $custom, $matches)) {
                    foreach ($matches[1] as $index => $url) {
                        $this->loaded_js[$url] = 1;
                        $custom = str_replace($matches[0][$index], '', $custom);
                    }
                }
                $custom = trim($custom);
            }
        }
    }

    /**
     * @param $headerstuff array
     */
    private function generateHead(&$headerstuff)
    {
        $includeCss = $headerstuff['styleSheets'];
        $includeJs = $headerstuff['scripts'];
        $embedCustom =& $headerstuff['custom'];

        $embed = '';

        $isAjaxRequest = isset($_SERVER['HTTP_X_REQUESTED_WITH']) && $_SERVER['HTTP_X_REQUESTED_WITH'] === 'XMLHttpRequest';

        if ($isAjaxRequest) {
            foreach ($headerstuff['preload_styleSheets'] as $url => $params) {
                $this->loaded_css[$url] = 1;
            }
            foreach ($headerstuff['preload_scripts'] as $url => $params) {
                $this->loaded_js[$url] = 1;
            }

            $embedCss = isset($headerstuff['style']['text/css']) ? $headerstuff['style']['text/css'] : null;
            $embedJs = isset($headerstuff['script']['text/javascript']) ? $headerstuff['script']['text/javascript'] : null;

            $headerstuff['preload_styleSheets'] = array();
            $headerstuff['preload_scripts'] = array();
            $headerstuff['styleSheets'] = array();
            $headerstuff['style'] = array();
            $headerstuff['scripts'] = array();
            $headerstuff['script'] = array();
            $headerstuff['custom'] = array();

            $jsMode = false;
            if (!empty($includeCss)) {
                // @todo: if jQueryUI is loaded over AJAX, jQM css should be restored
                $embed .= "<script>\n";
                $jsMode = true;
                foreach ($includeCss as $url => $param) {
                    if (!$this->isExcludedCss($url)) {
                        $embed .= "jqm.mj.loadCSS('" . addslashes(htmlspecialchars_decode($url)) . "');";
                        $this->loaded_css[$url] = 1;
                    }
                }
                $embed .= "\n";
            }

            if (!empty($embedCss)) {
                if ($jsMode) {
                    $embed .= '</script>';
                    $jsMode = false;
                }
                $embed .= '<style>' . (is_string($embedCss) ? $embedCss : implode('', $embedCss)) . '</style>';
            }

            if (!empty($includeJs) || !empty($embedJs)) {
                if (!$jsMode) {
                    $embed .= "<script>\n";
                    $jsMode = true;
                }
                if (!empty($includeJs)) {
                    foreach ($includeJs as $url => $param) {
                        if (!$this->isExcludedJs($url)) {
                            $embed .= "jqm.mj.loadJS('" . addslashes(htmlspecialchars_decode($url)) . "');";
                            $this->loaded_js[$url] = 1;
                        }
                    }
                    $embed .= "\n";
                }
                if (!empty($embedJs)) {
                    $embed .= (is_string($embedJs) ? $embedJs : implode("\n", $embedJs)) . "\n";
                }
            }

            if (!empty($embedCustom)) {
                foreach ($embedCustom as &$custom) {
                    $js = array();
                    if (preg_match_all('#<link\s[^>]*href="(([^">?]*)(?:\?[^">]*)?)"[^>]*/?>#', $custom, $matches)) {
                        foreach ($matches[0] as $index => $tag) {
                            if (strpos($tag, 'rel="stylesheet"') !== false) {
                                $url = $matches[1][$index];
                                if (!$this->isExcludedCss($url) && !$this->isExcludedCss($matches[2][$index])) {
                                    $js[] = "jqm.mj.loadCSS('" . htmlspecialchars_decode($url) . "');";
                                }
                                $custom = str_replace($tag, '', $custom);
                            }
                        }
                    }
                    if (preg_match_all('#<script\s[^>]*src="(([^">?]*)(?:\?[^">]*)?)"[^>]*></script>#', $custom, $matches)) {
                        foreach ($matches[1] as $index => $url) {
                            if (!$this->isExcludedJs($url) && !$this->isExcludedJs($matches[2][$index])) {
                                $js[] = "jqm.mj.loadJS('" . htmlspecialchars_decode($url) . "');";
                            }
                            $custom = str_replace($matches[0][$index], '', $custom);
                        }
                    }

                    if (count($js)) {
                        if (!$jsMode) {
                            $embed .= "<script>\n";
                            $jsMode = true;
                        }
                        $embed .= implode($js) . "\n";
                    }

                    $custom = trim($custom);
                    if (!empty($custom)) {
                        if ($jsMode) {
                            $embed .= '</script>';
                            $jsMode = false;
                        }
                        $embed .= $custom;
                    }
                }
                unset($custom);
            }

            if ($jsMode) {
                $embed .= '</script>';
            }
        }

        if (!$isAjaxRequest && !empty($embedCustom)) {
            foreach ($embedCustom as &$custom) {
                if (preg_match_all('#<link\s[^>]*href="(([^">?]*)(?:\?[^">]*)?)"[^>]*/?>#', $custom, $matches)) {
                    foreach ($matches[0] as $index => $tag) {
                        if (strpos($tag, 'rel="stylesheet"') !== false &&
                            ($this->isExcludedCss($matches[1][$index]) || $this->isExcludedCss($matches[2][$index]))
                        ) {
                            $custom = str_replace($tag, '', $custom);
                        }
                    }
                }
                if (preg_match_all('#<script\s[^>]*src="(([^">?]*)(?:\?[^">]*)?)"[^>]*></script>#', $custom, $matches)) {
                    foreach ($matches[1] as $index => $url) {
                        if ($this->isExcludedJs($url) || $this->isExcludedJs($matches[2][$index])) {
                            $custom = str_replace($matches[0][$index], '', $custom);
                        }
                    }
                }
            }
            unset($custom);
        }

        $head = $this->docHeadRender($headerstuff);

        $document = MJFactory::getDocument();
        $document->setBuffer($head, 'mjhead');
        $document->setBuffer($embed, 'mjbody');
    }

    /**
     * @param $headerstuff array
     * @return string
     */
    private function docHeadRender($headerstuff)
    {
        $document = MJFactory::getDocument();
        $buffer = '';

        include __DIR__ . '/head.php';

        return $buffer;
    }

    private function initURLs()
    {
        // set jQuery version
        switch ((string)$this->params->get('jqueryversion')) {
            case '17': // 1.7.2
                $this->jq_ver = '1.7.2';
                break;
            case '18': // 1.8.3
                $this->jq_ver = '1.8.3';
                break;
            case '19': // 1.9.1
                $this->jq_ver = '1.9.1';
                break;
            case '110': // 1.10.2
                $this->jq_ver = '1.10.2';
                break;
            case '111': // 1.11.3
                $this->jq_ver = '1.11.3';
                break;
            case '112': // 1.12.3
                $this->jq_ver = '1.12.3';
                break;
            case '20': // 2.0.3
                $this->jq_ver = '2.0.3';
                break;
            case '21': // 2.1.4
                $this->jq_ver = '2.1.4';
                break;
            case '22': // 2.2.4
                $this->jq_ver = '2.2.4';
                break;
            default: // 1.9.1
                $this->jq_ver = '1.9.1';
        }

        if ($this->params->get('jqmigrate')) {
            $this->jq_migration = true;
        }

        // check themeroller
        $themeroller = $this->params->get('theme');
        $loadJqmStructure = true;
        if ($themeroller) {
            $this->themeroller = $themeroller;
            $this->themeroller_css = $this->base . "themes/$themeroller/$themeroller";
            $this->themeroller_icons = $this->base . "themes/$themeroller/jquery.mobile.icons.min.css";
            $this->themeroller_custom = $this->base . "themes/$themeroller/custom";
            if (!file_exists(JPATH_ROOT . $this->themeroller_icons)) {
                $this->themeroller_icons = null;
            }
            if (!file_exists(JPATH_ROOT . $this->themeroller_custom . '.css')) {
                $this->themeroller_custom = null;
            }
            $theme_params = JPATH_ROOT . $this->base . "themes/$themeroller/$themeroller.ini";
            if (is_file($theme_params)) {
                $params = parse_ini_string(file_get_contents($theme_params));
                if (isset($params['include_structure']) && !$params['include_structure']) {
                    $loadJqmStructure = false;
                }
            }
        }

        // fix default module content theme
        if (!$this->params->get('theme_modulecontent')) {
            $content_theme = $this->params->get('theme_page');
            if (!$content_theme) {
                $content_theme = 'a';
            }
            $this->params->set('theme_modulecontent', $content_theme);
        }

        $jqm_ver = $this->jqm_ver;
        $jq_ver = $this->jq_ver;
        $jqmigr_ver = $this->jq_migration_ver;

        if ($this->params->get('load_external')) {
            include __DIR__ . '/cdn.php';
            /** @var array $cdnList */

            $cdn = $this->params->get('load_external');
            if (!isset($cdnList[$cdn])) {
                $cdn = '1';
            }

            $protocol = (!empty($_SERVER['HTTPS']) && (strtolower($_SERVER['HTTPS']) !== 'off')) ? 'https:' : 'http:';

            if ($cdn === '1') { // auto (value '1' is for backward compatibility)
                $cdn = 'jquery.com';
//              $cdn = ($protocol==='https:') ? 'aspnetcdn.com' : 'jquery.com';
            }

            $jqm_css_file = $themeroller ? 'jquery.mobile.structure.css' : 'jquery.mobile.css';

            $this->jqm_jq = $protocol . str_replace('*', $jq_ver, $cdnList[$cdn]['jquery.js']);
            $this->jqm_css = $loadJqmStructure ? $protocol . str_replace('*', $jqm_ver, $cdnList[$cdn][$jqm_css_file]) : '';
            $this->jqm_jqm = $protocol . str_replace('*', $jqm_ver, $cdnList[$cdn]['jquery.mobile.js']);

            if ($this->jq_migration) {
                $this->jqm_jqmigr = $protocol . str_replace('*', $jqmigr_ver, $cdnList[$cdn]['jquery.migrate.js']);
            }
        } else {
            $jqm_url = $this->base . 'vendor/jqm/';

            $this->jqm_jq = $jqm_url . 'jquery-' . $jq_ver;
            $this->jqm_css = $loadJqmStructure ? $jqm_url . 'jquery.mobile' . ($themeroller ? '.structure-' : '-') . $jqm_ver : '';
            $this->jqm_jqm = $jqm_url . 'jquery.mobile-' . $jqm_ver;

            if ($this->jq_migration) {
                $this->jqm_jqmigr = $jqm_url . 'jquery-migrate-' . $jqmigr_ver;
            }
        }
    }

    private function forceMootools(&$headerstuff)
    {
        $mootoolsFound = false;

        $scripts = $headerstuff['scripts'];
        if (!is_array($scripts)) {
            $scripts = array();
        }

        $regex = '#mootools-(?:core|more)(?:-uncompressed)?\.js#';
        foreach ($scripts as $url => $meta) {
            if (preg_match($regex, $url)) {
                $mootoolsFound = true;
                break;
            }
        }

        if (!$mootoolsFound) {
            $mootools_js = MJHtml::_('script', 'system/mootools-core.js', false, true, true, false);
            // trick for array_unshift with key preserving
            $scripts = array_reverse($scripts, true);
            $scripts[$mootools_js] = array('mime' => 'text/javascript', 'defer' => false, 'async' => false);
            $scripts = array_reverse($scripts, true);
            $headerstuff['scripts'] = $scripts;
        }
    }

    private function combinerInit()
    {
        $this->combinerSource = array();
    }

    private function combinerAdd($type, $text)
    {
        switch ($type) {
            case 'cssfile':
                if ($this->isExcludedCss($text)) {
                    return;
                }
                break;
            case 'jsfile':
                if ($this->isExcludedJs($text)) {
                    return;
                }
                break;
        }
        $this->combinerSource[] = array('type' => $type, 'text' => $text);
    }

    private function combinerGetHash()
    {
        $hash = '';
        foreach ($this->combinerSource as $src) {
            switch ($src['type']) {
                case 'cssfile':
                    $file = $this->htmlGetFullPath('css', $src['text']);
                    $hash .= $src['text'] . '|' . @filemtime(JPATH_ROOT . $file) . '|';
                    break;
                case 'jsfile':
                    $file = $this->htmlGetFullPath('js', $src['text']);
                    $hash .= $src['text'] . '|' . @filemtime(JPATH_ROOT . $file) . '|';
                    break;
                case 'css':
                case 'js':
                    $hash .= $src['text'] . '|';
                    break;
            }
        }
        return substr(sha1($hash), 0, 8);
    }

    private function htmlGetFullPath($type, $base)
    {
        if (substr($base, -4) === '.css' || substr($base, -3) === '.js' || strpos($base, '/') === false) {
            return $base;
        }

        $absoluteUri = (strncmp($base, 'http', 4) === 0);

        if ($this->rtl && !$absoluteUri && file_exists(JPATH_ROOT . $base . '.rtl.' . $type)) {
            $base .= '.rtl';
        }

        if (!$this->debug) {
            $full_min = $base . '.min.' . $type;
            if ($absoluteUri || file_exists(JPATH_ROOT . $full_min)) {
                return $full_min;
            }
        }

        return $base . '.' . $type;
    }

    private function htmlCSS($base)
    {
        $url = $this->htmlGetFullPath('css', $base);
        if ($this->isExcludedCss($url)) {
            return '';
        }
        $this->loaded_css[$url] = 1;
        if (strpos($url, '//') === false) {
            $url = $this->baseurl . $url;
        }
        return '<link rel="stylesheet" href="' . $url . '">';
    }

    private function htmlJS($base)
    {
        $url = $this->htmlGetFullPath('js', $base);
        if ($this->isExcludedJs($url)) {
            return '';
        }
        $this->loaded_js[$url] = 1;
        if (strpos($url, '//') === false) {
            $url = $this->baseurl . $url;
        }
        return '<script src="' . $url . '"></script>';
    }

    private function combinerGetDebug()
    {
        $buffer = '';
        foreach ($this->combinerSource as $src) {
            switch ($src['type']) {
                case 'cssfile':
                    $buffer .= $this->htmlCSS($src['text']);
                    break;
                case 'jsfile':
                    $buffer .= $this->htmlJS($src['text']);
                    break;
                case 'css':
                    $buffer .= '<style ress-merge>' . trim($src['text']) . '</style>';
                    break;
                case 'js':
                    $buffer .= "<script ress-merge>\n" . trim($src['text']) . "\n</script>";
                    break;
            }
        }
        return $buffer;
    }

    private function combinerFixCSSURL($content, $url)
    {
        $base = dirname($url) . '/';
        return preg_replace('#\burl\(\s*?(?:["\'](?!/|\w+?:)|(?![/"\']|\w+?:))#', '\\0' . $this->baseurl . $base, $content);
    }

    private function combinerGetRelease()
    {
        $buffer = '';
        foreach ($this->combinerSource as $src) {
            $content = '';
            switch ($src['type']) {
                case 'cssfile':
                    $url = $this->htmlGetFullPath('css', $src['text']);
                    if (!$this->isExcludedCss($url)) {
                        $file = JPATH_ROOT . $url;
                        $content = file_get_contents($file);
                        $content = $this->combinerFixCSSURL($content, $url);
                        $this->loaded_css[$url] = 1;
                    }
                    break;
                case 'jsfile':
                    $url = $this->htmlGetFullPath('js', $src['text']);
                    if (!$this->isExcludedJs($url)) {
                        $file = JPATH_ROOT . $url;
                        $content = trim(file_get_contents($file));
                        if ($content !== '' && !in_array(substr($content, -1), array('}', ';'), true)) {
                            $content .= ';';
                        }
                        $this->loaded_js[$url] = 1;
                    }
                    break;
                case 'css':
                case 'js':
                    $content = $src['text'];
                    break;
            }
            $buffer .= $content . "\n";
        }
        return $buffer;
    }

    private function combinerGet($type)
    {
        jimport('joomla.filesystem.file');
        jimport('joomla.filesystem.folder');
        if (!$this->debug && $this->params->get('combine')) {
            $hash = $this->combinerGetHash();
            $cache_dir = JPATH_THEMES . '/' . $this->template . '/cache';
            $cached_file = $cache_dir . '/' . $hash . '.' . $type;
            $cached_url = $this->baseurl . '/' . ltrim(str_replace(JPATH_ROOT, '', $cached_file), '/\\');

            $success = true;
            if (!file_exists($cached_file)) {
                $content = $this->combinerGetRelease();

                if (!MJFile::write($cached_file, $content)) {
                    MJFile::delete($cached_file);
                    $success = false;
                }

                $cached_file_gz = $cached_file . '.gz';
                $content = gzencode($content, 9);
                if (!MJFile::write($cached_file_gz, $content)) {
                    MJFile::delete($cached_file_gz);
                }

                $files = MJFolder::files($cache_dir, '\.' . $type . '(?:\.gz)?$');
                $exclude = array($cached_file, $cached_file_gz);
                foreach ($files as $file) {
                    $full = $cache_dir . '/' . $file;
                    if (!in_array($full, $exclude, true)) {
                        MJFile::delete($full);
                    }
                }
            }

            if ($success) {
                switch ($type) {
                    case 'css':
                        return '<link rel="stylesheet" href="' . $cached_url . '">';
                    case 'js':
                        return '<script src="' . $cached_url . '"></script>';
                }
            }
        }

        return $this->combinerGetDebug();
    }

    /**
     * @return string
     * @info Called from head.php
     */
    private function loadCSS()
    {
        $html = '';
        $this->combinerInit();

        if ($this->params->get('load_external') && $this->jqm_css) {
            if ($this->themeroller_icons) {
                $html .= $this->htmlCSS($this->themeroller_icons);
            }
            if ($this->themeroller_css) {
                $html .= $this->htmlCSS($this->themeroller_css);
            }
            $html .= $this->htmlCSS($this->jqm_css);
        } else {
            if ($this->themeroller_icons) {
                $this->combinerAdd('cssfile', $this->themeroller_icons);
            }
            if ($this->themeroller_css) {
                $this->combinerAdd('cssfile', $this->themeroller_css);
            }
            if ($this->jqm_css) {
                $this->combinerAdd('cssfile', $this->jqm_css);
            }
        }

        if ($this->rtl) {
            $this->combinerAdd('cssfile', $this->base . 'css/rtl');
        }

        $this->combinerAdd('cssfile', $this->base . 'css/structure');
        if (is_file(JPATH_ROOT . $this->base . 'css/fix.css')) {
            $this->combinerAdd('cssfile', $this->base . 'css/fix');
        }
        if ($this->themeroller_custom) {
            $this->combinerAdd('cssfile', $this->themeroller_custom);
        }
        $this->combinerAdd('cssfile', $this->base . 'css/mj');

        $backcolor = $this->params->get('backcolor');
        $backtile = $this->params->get('backtile');
        if ($backtile === '-') {
            $backtile = '';
        }
        $backpadding = $this->params->get('backpadding');
        $opacity = $this->params->get('opacity');

        $homebackcolor = $this->params->get('homebackcolor');
        $homebacktile = $this->params->get('homebacktile');
        if ($homebacktile === '-') {
            $homebacktile = '';
        }
        $homebackpadding = $this->params->get('homebackpadding');
        $homeopacity = $this->params->get('homeopacity');

        $css_uipage = '';
        $css_uipagehome = '';
        $css_content = '';
        $css_contenthome = '';

        if (!empty($backcolor)) {
            $css_uipage .= "background-color:#$backcolor;";
        }
        if (!empty($homebackcolor)) {
            $css_uipagehome .= "background-color:#$homebackcolor;";
        }

        if (!empty($backtile)) {
            if (strpos($backtile, '#') !== false) {
                $backtile = explode('#', $backtile, 2)[0];
            }
            if (strpos($backtile, '://') === false) {
                $backtile = MJUri::root(true) . '/' . $backtile;
            }
            $css_uipage .= "background-image:url('$backtile');";
        }
        if (!empty($homebacktile)) {
            if (strpos($homebacktile, '#') !== false) {
                $homebacktile = explode('#', $homebacktile, 2)[0];
            }
            if (strpos($homebacktile, '://') === false) {
                $homebacktile = MJUri::root(true) . '/' . $homebacktile;
            }
            $css_uipagehome .= "background-image:url('$homebacktile');";
        }

        if (!empty($backpadding)) {
            $css_content .= 'padding-top:' . max((float)$backpadding, 0.01) . '%;';
        }
        if (!empty($homebackpadding)) {
            $css_contenthome .= 'padding-top:' . max((float)$homebackpadding, 0.01) . '%;';
        }

        if (!empty($opacity) && $opacity < 100) {
            $css_content .= 'opacity:' . (0.01 * (float)$opacity) . ';';
        }
        if (!empty($homeopacity) && $homeopacity < 100) {
            $css_contenthome .= 'opacity:' . (0.01 * (float)$homeopacity) . ';';
        }

        $overrides = '';

        if (!empty($css_uipage) || !empty($css_uipagehome) || !empty($css_content) || !empty($css_contenthome)) {
            $css_content = 'position:relative;background-color:transparent;' . $css_content;

            $css_uipage = rtrim($css_uipage, ';');
            $css_uipagehome = rtrim($css_uipagehome, ';');
            $css_content = rtrim($css_content, ';');
            $css_contenthome = rtrim($css_contenthome, ';');

            if (!empty($css_uipage)) {
                $overrides .= '.ui-page,.ui-page>.ui-panel-wrapper,.ui-page>.ui-panel{' . $css_uipage . '}';
            }
            if (!empty($css_uipagehome) && $css_uipagehome !== $css_uipage) {
                $overrides .= '.ui-page.home,.ui-page.home>.ui-panel-wrapper,.ui-page.home>.ui-panel{' . $css_uipagehome . '}';
            }
            if (!empty($css_content)) {
                $overrides .= '.ui-content{' . $css_content . '}';
            }
            if (!empty($css_contenthome) && $css_contenthome !== $css_content) {
                $overrides .= '.ui-page.home .ui-content{' . $css_contenthome . '}';
            }
        }

        if ($overrides) {
            $this->combinerAdd('css', $overrides);
        }

        $html .= $this->combinerGet('css');

        return $html;
    }

    /**
     * @return string
     * @info Called from head.php
     */
    private function loadJS()
    {
        $load_external = $this->params->get('load_external');
        $ajax = $this->params->get('ajax');
        $transition = $this->params->get('transition', 'fade');
        $theme_header = $this->params->get('theme_header');
        $enhance = $this->params->get('enhance');

        $html = '';
        $this->combinerInit();

        if ($load_external || $this->compatJQueryEasy) {
            $html .= $this->htmlJS($this->jqm_jq);
            if ($this->jq_migration) {
                $html .= $this->htmlJS($this->jqm_jqmigr);
            }
        } else {
            $this->combinerAdd('jsfile', $this->jqm_jq);
            if ($this->jq_migration) {
                $this->combinerAdd('jsfile', $this->jqm_jqmigr);
            }
        }

        $mobileinit = array();
//        $mobileinit[] = 'jQuery.mobile.loadingMessage=false;';
        $errorMessage = MJText::_('TPL_MOBILE_JQM__PAGELOADERROR');
        if ($errorMessage !== 'Error Loading Page') {
            $mobileinit[] = 'jQuery.mobile.pageLoadErrorMessage="' . addslashes($errorMessage) . '";';
        }
        if (!$ajax) {
            // ??? hashListeningEnabled is necessary for panel and popups
            $mobileinit[] = 'jQuery.mobile.ajaxEnabled=false;jQuery.mobile.pushStateEnabled=false;';//jQuery.mobile.hashListeningEnabled=false;
        } else {
            // disable buggy URL rebase
            $mobileinit[] = 'jQuery.mobile.dynamicBaseEnabled=false;';
        }
        if ($transition !== 'fade' && $transition !== '') {
            $mobileinit[] = 'jQuery.mobile.defaultPageTransition="' . $transition . '";';
        }
        if ($enhance) {
            $mobileinit[] = 'jQuery.mobile.ignoreContentEnabled=true;';
        }
        if ($this->rtl) {
            $mobileinit[] = 'jQuery.mobile.behaviours.addFirstLastClasses.options.icon="carat-r";';
            $mobileinit[] = 'jQuery.mobile.behaviours.addFirstLastClasses.options.splitIcon="carat-r";';
        }

        $mobileinit[] = 'if(!jQuery.mobile.gradeA()){(document.documentElement||document.body).className="";}';

        $mobileinit = count($mobileinit) ? ("jQuery(document).bind('mobileinit',function(){" . implode('', $mobileinit) . "});") : '';
        /** @var string $mobileinit */
        if ($load_external) {
            if (!empty($mobileinit)) {
                $html .= "<script>{$mobileinit}</script>";
            }
            $html .= $this->htmlJS($this->jqm_jqm);
        } else {
            if (!empty($mobileinit)) {
                $this->combinerAdd('js', $mobileinit);
            }
            $this->combinerAdd('jsfile', $this->jqm_jqm);
        }

        $postinit = ($this->params->get('removejquery') && !$this->params->get('mootools')) ?
            'jqm=jQuery;' :
            // trick to bypass jQueryEasy regexp
            'jqm=jQuery.noConflict(' . ($this->compatJQueryEasySafeNoConflict ? ' ' : '') . ');';
        if ($theme_header) {
            $postinit .= "jqm.mobile.page.prototype.options.headerTheme='$theme_header';";
        }
        $this->combinerAdd('js', $postinit);

        $this->combinerAdd('jsfile', $this->base . 'js/pageinit');
        if ($ajax) {
            $this->combinerAdd('jsfile', $this->base . 'js/mj-ajax');
        }

        $custom_js = $this->path . '/js/custom.js';
        if (is_file($custom_js) && filesize($custom_js)) {
            $this->combinerAdd('jsfile', $this->base . 'js/custom.js');
        }

        $html .= $this->combinerGet('js');

        return $html;
    }

    /**
     * @param Ressio $ressio
     */
    public function onMJRessioPrepare($ressio)
    {
        if ($this->params->get('ajax')) {
            $ressio->config->js->mergeheadbody = false;
            $dispatcher = $ressio->di->dispatcher;
            // @todo Check these events are not renamed
            $dispatcher->addListener('JsFileMinifyBefore', array($this, 'onRessioJsFileMinifyBefore'));
            $dispatcher->addListener('JsCombineAfter', array($this, 'onRessioJsCombineAfter'));
        }
    }

    /**
     * @param Ressio_Event $event
     * @param string $url
     * @param string $content
     */
    public function onRessioJsFileMinifyBefore($event, $url, &$content)
    {
        $content = rtrim($content, ";\n\r") . ";mj.push('" . addcslashes($url, "'") . "');";
    }

    /**
     * @param Ressio_Event $event
     * @param string $js
     */
    public function onRessioJsCombineAfter($event, &$js)
    {
        if (strpos($js, 'mj.push') !== false) { // skip embedded scripts
            $js = 'var mj=mj||[];' . $js;
        }
    }

    /**
     * @param string $url
     * @return bool
     */
    private function isExcludedCss($url)
    {
        return isset($this->loaded_css[$url]) || ($this->exclude_css !== '' && preg_match('/^' . $this->exclude_css . '$/', $url));
    }

    /**
     * @param string $url
     * @return bool
     */
    private function isExcludedJs($url)
    {
        return isset($this->loaded_js[$url]) || ($this->exclude_js !== '' && preg_match('/^' . $this->exclude_js . '$/', $url));
    }
}
