<?php
/**
 * Mobile Basic template for Mobile Joomla! extension
 * https://www.mobilejoomla.com
 *
 * @version        3.2.2
 * @license        GNU/GPL v2 - http://www.gnu.org/licenses/gpl-2.0.html
 * @copyright    (C) 2008-2024 Kuneri Ltd. / Denis Ryabov
 * @date        February 2024
 */
defined('_JEXEC') or die('Restricted access');

class_exists('JPluginHelper') || JLoader::registerAlias('JPluginHelper', Joomla\CMS\Plugin\PluginHelper::class);
class_exists('JFile') || JLoader::registerAlias('JFile', Joomla\Filesystem\File::class);

class mobile_basicInstallerScript
{
    /**
     * @param string $route
     * @param JInstallerComponent $adapter
     * @return bool|void
     */
    public function preflight($route, $adapter)
    {
        // @todo: J!2.5-3.3 doesn't support <scriptfile> tag (BUT package does, is it ok to repack template as a package?)
        if ($route === 'install') {
            if (version_compare(JVERSION, '4.0', '<') || version_compare(JVERSION, '5.0', '>=')) {
                throw new RuntimeException('Inappropriate Joomla version.');
            }

            if (!JPluginHelper::isEnabled('system', 'mobilejoomla')) {
                throw new RuntimeException('This template requires the Mobile Joomla! extension.');
            }

            $xml = simplexml_load_file(JPATH_ADMINISTRATOR . '/components/com_mobilejoomla/mobilejoomla.xml');
            if (version_compare($xml->version, '2.999', '<')) {
                throw new RuntimeException('This template requires the Mobile 3.0 (or higher) Joomla! extension.');
            }
        }
    }

    /**
     * @param string $route
     * @param JInstallerPackage $adapter
     */
    public function postflight($route, $adapter)
    {
        if ($route !== 'uninstall') {
            $adapter->getParent()->message = '';
            $this->doInstall();
            // @todo load language file for strings?
            ?>
            <link rel="stylesheet" type="text/css" href="../media/com_mobilejoomla/css/mj.bootstrap.min.css">
            <style>
                #mj .list-group-action {
                    padding-right: 2rem;
                }

                #mj .list-group-action::after {
                    position: absolute;
                    content: '❯';
                    text-align: center;
                    right: .5rem;
                    width: 1em;
                    top: 50%;
                    height: 2rem;
                    line-height: 2rem;
                    margin-top: -1rem;
            </style>
            <div id="mj">
                <div class="well">
                    <h1 class='fw-bold text-success'>
                        Mobile Basic 3.2.2
                    </h1>
                    <p>Mobile Basic template for Mobile Joomla! extension</p>
                    <div class="list-group">
                        <?php if (JPluginHelper::isEnabled('mobile', 'adaptive')): ?>
                            <a class="list-group-item list-group-action btn-outline-info"
                               href="index.php?option=com_mobilejoomla&controller=settings&view=adaptive">
                                Open <b>Mobile's Adaptive settings</b> to select this template for mobile version
                            </a>
                            <span class="list-group-item list-group-item-secondary disabled py-0">or</span>
                        <?php endif; ?>
                        <a class="list-group-item list-group-action btn-outline-info"
                           href="index.php?option=com_templates&view=styles&client_id=0">
                            Open Joomla's <b>Template Styles</b> page to set this template as default
                        </a>
                    </div>
                </div>
            </div>
            <?php
        }

        return true;
    }

    private function doInstall()
    {
        $template_path = JPATH_ROOT . '/templates/mobile_basic/';

        $this->createUserFiles($template_path);
        $this->updateUnmodifiedSysFiles($template_path);
        $this->removeOldFiles($template_path);
    }

    /**
     * @param string $template_path
     * @return void
     */
    private function createUserFiles($template_path)
    {
        $names = array(
            'css/custom.css',
            'css/custom_preload.txt',
            'js/custom.js',
            'js/custom_preload.txt',
        );
        $data = '';
        foreach ($names as $name) {
            if (!is_file($template_path . $name)) {
                JFile::write($template_path . $name, $data);
            }
        }
    }

    /**
     * @param string $template_path
     * @return void
     */
    private function updateUnmodifiedSysFiles($template_path)
    {
        $names = array(
            '.htaccess' => array('56868306b03ba249b6bc4dc4d2912940', '57372b279ee1ccdafb16837339f9036b'),
            'favicon.ico' => array('4682009df58c0fe58ee462247355de56'),
            'logo.png' => array('c74492c3b4d777f4bd30157d95a757b8', '562d4fefcd3be5fc2b7d92a9004aa07b'),
        );

        $installed_flag = $template_path . '/media/installed.flag';
        if (is_file($installed_flag)) {
            if (!is_file($template_path . '.htaccess')) {
                unset($names['.htaccess']);
            }
        } else {
            JFile::write($installed_flag, $data);
        }

        foreach ($names as $name => $hashes) {
            $file = $template_path . $name;
            if (is_file($file) && !in_array(md5_file($file), $hashes, true)) {
                return;
            }
        }

        foreach ($names as $name => $hashes) {
            $src = $template_path . 'media/' . $name;
            if (is_file($src)) {
                $dest = $template_path . $name;
                if (is_file($dest)) {
                    JFile::delete($dest);
                }
                JFile::copy($src, $dest);
            }
        }
    }

    /**
     * @param string $template_path
     * @return void
     */
    private function removeOldFiles($template_path)
    {
        $files = new RegexIterator(
            new RecursiveIteratorIterator(new RecursiveDirectoryIterator($template_path)),
            '/\.php$/'
        );
        $version_regex = '/@version\s+' . preg_quote('3.2.2', '/') . '/';
        foreach ($files as $file) {
            $path = $file->getRealPath();
            $content = file_get_contents($path);
            if (
                    strpos($content, 'Mobile Basic template for Mobile Joomla! extension') !== false &&
                    strpos($content, '@version') !== false &&
                    !preg_match($version_regex, $content)
            ) {
                @unlink($path);
            }
        }
    }
}